/***********************************************************
 * $Id$
 *
 * FancyLibs qmi wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQMIWDSCLIENT_H_
#define FANCYQMIWDSCLIENT_H_

#include <fancyqmienums.h>
#include <fancyobject.h>

extern "C" {
typedef struct _QmiClientWds QmiClientWds;
}

namespace Fancy {
namespace QMI {

class QmiDevice;
class QmiAsyncHandler;
class QmiStartNetworkInput;
class QmiStartNetworkOutput;
class QmiPacketServiceStatusCallback;

/**
 * A client operating on the Device Management Service of a device.
 *
 * A client instance is normally returned as the result of
 * an @c QmiDevice::allocateWdsClient() asynchronous operation.
 */
class FANCYQMI_API QmiWdsClient : virtual public Fancy::Object
{
	Fancy::ObjPtr<QmiDevice> device;
	QmiClientWds *clientWds;

public:
	typedef Fancy::ObjPtr<QmiWdsClient> ptr_t;

	/**
	 * Instantiate from a native glib instance.
	 * The instance is dereferenced in the destructor but not
	 * additionally referenced in the constructor.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<QmiDevice> &device, QmiClientWds *clientWds);

	/**
	 * Open a network connection.
	 *
	 * @param input The operating mode to set.
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       operation mode to be set.
	 * @param result An asynchronous result handler, which receives a
	 *               Fancy::ObjPtr<Fancy::QMI::QmiStartNetworkOutput> instance.
	 */
	void startNetwork(const Fancy::ObjPtr<QmiStartNetworkInput> &input, int timeoutSeconds,  const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Stop a network connection
	 *
	 * @param packetDataHandle the handle from QmiStartNetworkOutput returned as result from startNetwork()
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       operation mode to be set.
	 * @param result An asynchronous result handler, which receives a
	 *               Fancy::ObjPtr<Fancy::Boolean> instance indicating shutdown success.
	 */
	void stopNetwork(unsigned packetDataHandle, int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Register a callback packet service status callback events from the device.
	 */
	void setPacketServiceStatusCallback(const Fancy::ObjPtr<QmiPacketServiceStatusCallback> &cb);

	/**
	 * Register this client for release at the underlying device.
	 * The stop of the main loop will be postponed until
	 * all release operations are finished.
	 *
	 * If not performed explicitly, this operation is performed with
	 * a timeout of 1 seconds by the destructor.
	 *
	 * This operation renders the client instance unusable and set the
	 * kept device pointer to null.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *               client to become released.
	 */
	void release(int timeoutSeconds);

	std::string toString() const;

protected:

	QmiWdsClient(const Fancy::ObjPtr<QmiDevice> &device, QmiClientWds *clientWds);
	virtual ~QmiWdsClient();
};

}} // end namespaces

#endif
